@@ -1,7 +1,7 @@
 
 /* SLgtk:  S-Lang bindings for the GTK+ widget set {{{
  *
- * Copyright (C) 2003-2004 Massachusetts Institute of Technology 
+ * Copyright (C) 2003-2009 Massachusetts Institute of Technology 
  * Copyright (C) 2002 Michael S. Noble <mnoble@space.mit.edu>
  *
  * This software was partially developed by the MIT Center for Space
@@ -64,6 +64,13 @@
 
 #include "slgtkicons.h"
 
+static const unsigned int _gtk_version = 10000*GTK_MAJOR_VERSION+ 
+				100*GTK_MINOR_VERSION + GTK_MICRO_VERSION;
+
+#if GTK_MAJOR_VERSION <= 2 && GTK_MINOR_VERSION < 14
+#define GTK_OLDER_THAN_214 1
+#endif
+
 const int	slgtk_version	  = SLGTK_VERSION;
 const char	*slgtk_version_string = SLGTK_VERSION_STRING;
 int		slgtk_debug;
@@ -212,7 +219,7 @@ static int execl_cmd(char *path, char *arg0, char *arg1, char* arg2)
 
 /* GtkTextIter interface {{{ */
 
-static void sl_gtk_text_buffer_get_iter_at_line_offset (
+static void sl_gtk_text_buffer_get_iter_at_line_offset ( /*{{{*/
 				Slirp_Opaque *textbuf,
                                 gint *line_number,
 				gint *char_offset)
@@ -224,9 +231,9 @@ static void sl_gtk_text_buffer_get_iter_at_line_offset (
 	      					*line_number,*char_offset);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_get_iter_at_offset (
+static void sl_gtk_text_buffer_get_iter_at_offset ( /*{{{*/
 				Slirp_Opaque *textbuf,
 				gint *char_offset)
 {
@@ -237,9 +244,9 @@ static void sl_gtk_text_buffer_get_iter_at_offset (
 	      						*char_offset);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_get_iter_at_line (
+static void sl_gtk_text_buffer_get_iter_at_line ( /*{{{*/
 				Slirp_Opaque *textbuf,
                                 gint *line_number)
 {
@@ -250,9 +257,9 @@ static void sl_gtk_text_buffer_get_iter_at_line (
 	      						*line_number);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_get_start_iter  ( Slirp_Opaque *textbuf)
+static void sl_gtk_text_buffer_get_start_iter  ( Slirp_Opaque *textbuf) /*{{{*/
 {
    GtkTextIter *iter;
  
@@ -260,9 +267,9 @@ static void sl_gtk_text_buffer_get_start_iter  ( Slirp_Opaque *textbuf)
 	gtk_text_buffer_get_start_iter(textbuf->instance, iter);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_get_end_iter (Slirp_Opaque *textbuf)
+static void sl_gtk_text_buffer_get_end_iter (Slirp_Opaque *textbuf) /*{{{*/
 {
    GtkTextIter *iter;
  
@@ -270,9 +277,9 @@ static void sl_gtk_text_buffer_get_end_iter (Slirp_Opaque *textbuf)
 	gtk_text_buffer_get_end_iter(textbuf->instance, iter);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_get_iter_at_mark (Slirp_Opaque *textbuf,
+static void sl_gtk_text_buffer_get_iter_at_mark (Slirp_Opaque *textbuf, /*{{{*/
       		Slirp_Opaque *mark)
 {
    GtkTextIter *iter;
@@ -282,7 +289,7 @@ static void sl_gtk_text_buffer_get_iter_at_mark (Slirp_Opaque *textbuf,
 							mark->instance);
 
    SLang_push_opaque(GtkTextIter_Type, iter, 1);
-}
+} /*}}}*/
 
 #if 0
 void        gtk_text_buffer_get_iter_at_line_index
@@ -299,7 +306,40 @@ void        gtk_text_buffer_get_bounds      (GtkTextBuffer *buffer,
                                              GtkTextIter *end);
 #endif
 
-static void sl_gtk_text_buffer_create_tag(void)
+static void sl_gtk_text_iter_forward_search (void) /*{{{*/
+{
+   const GtkTextIter* from;		/* where to start search in buffer */
+   Slirp_Opaque* from_o = NULL;
+   GtkTextSearchFlags flags;		/* how to conduct the search  */
+   GtkTextIter *begin, *end;		/* markers delimiting a match */
+   gboolean found;			/* whether a match was found  */
+   gchar* text;				/* text we're looking for     */
+
+   if (SLang_Num_Function_Args != 3 ||
+	SLang_pop_int((int*)&flags) == -1 ||
+	SLang_pop_string((char**)&text) == -1 ||
+	SLang_pop_opaque(GtkTextIter_Type, (void**)&from, &from_o) == -1 )
+	usage_err(99,"(begin,end) = gtk_text_iter_forward_search(from,text,flags)");
+
+   if ( (begin = (GtkTextIter *)SLmalloc(sizeof(GtkTextIter))) == NULL ||
+	(end   = (GtkTextIter *)SLmalloc(sizeof(GtkTextIter))) == NULL)
+	return;
+
+   /* FIXME: remove restrictive use of NULL == search until end of buffer */
+   found = gtk_text_iter_forward_search(from, text, flags, begin, end, NULL);
+
+   SLang_free_opaque(from_o);
+   SLang_free_slstring(text);
+
+   if (!found)
+	begin = end = NULL;
+
+   SLang_push_opaque(GtkTextIter_Type, begin, 1);
+   SLang_push_opaque(GtkTextIter_Type, end, 1);
+
+} /*}}}*/
+
+static void sl_gtk_text_buffer_create_tag(void) /*{{{*/
 {
    GParamSpec *pspec;
    Slirp_Opaque *textbuf_o = NULL;
@@ -361,9 +401,9 @@ static void sl_gtk_text_buffer_create_tag(void)
 	   SLang_push_opaque (GObject_Type, tag, 0);
 	else
 	   SLang_push_null();
-}
+} /*}}}*/
 
-static void sl_gtk_text_buffer_insert_with_tags(void)
+static void sl_gtk_text_buffer_insert_with_tags(void) /*{{{*/
 {
 
    int len, start_offset;
@@ -406,7 +446,7 @@ static void sl_gtk_text_buffer_insert_with_tags(void)
 	SLang_free_opaque(textbuf_o);
 	SLang_free_opaque(iter_o);
 	SLang_free_slstring(text);
-}
+} /*}}}*/
 /* }}} */
 
 /* GtkMenu interface {{{ */
@@ -961,7 +1001,7 @@ static void sl_gtk_adjustment_get_value (Slirp_Opaque* o)
    (void) SLang_push_float( GTK_ADJUSTMENT (o->instance)->value);
 }
 
-
+#ifdef GTK_OLDER_THAN_214
 static void sl_gtk_adjustment_get_lower (Slirp_Opaque* o)
 {
    (void) SLang_push_float( GTK_ADJUSTMENT (o->instance)->lower);
@@ -990,6 +1030,7 @@ static void sl_gtk_adjustment_get_page_size (Slirp_Opaque* o)
 {
    (void) SLang_push_float( GTK_ADJUSTMENT (o->instance)->page_size);
 }
+#endif
 /* }}} */
 
 /* Intrinsic function and variable tables {{{ */
@@ -1034,11 +1075,13 @@ static SLang_Intrin_Fun_Type GtkObject_Funcs[] =
   MAKE_INTRINSIC_1("gtk_object_type",sl_gtk_object_type,V,O),
   MAKE_INTRINSIC_1("gtk_object_type_name",sl_gtk_object_type_name,V,O),
   MAKE_INTRINSIC_1("gtk_adjustment_get_value",sl_gtk_adjustment_get_value,V,O),
+#ifdef GTK_OLDER_THAN_214
   MAKE_INTRINSIC_1("gtk_adjustment_get_lower",sl_gtk_adjustment_get_lower, V,O),
   MAKE_INTRINSIC_1("gtk_adjustment_get_upper",sl_gtk_adjustment_get_upper, V,O),
   MAKE_INTRINSIC_1("gtk_adjustment_get_step_increment",sl_gtk_adjustment_get_step_increment, V, O),
   MAKE_INTRINSIC_1("gtk_adjustment_get_page_increment",sl_gtk_adjustment_get_page_increment, V, O),
   MAKE_INTRINSIC_1("gtk_adjustment_get_page_size",sl_gtk_adjustment_get_page_size, V, O),
+#endif
 
   SLANG_END_INTRIN_FUN_TABLE
 };
@@ -1057,6 +1100,8 @@ static SLang_Intrin_Fun_Type GObject_Funcs[] =
 		  	sl_gtk_text_buffer_get_start_iter,V,O),
   MAKE_INTRINSIC_1("gtk_text_buffer_get_end_iter",
 		  	sl_gtk_text_buffer_get_end_iter,V,O),
+  MAKE_INTRINSIC_0("gtk_text_iter_forward_search",
+			sl_gtk_text_iter_forward_search,V),
   MAKE_INTRINSIC_0("gtk_text_buffer_create_tag",
 			sl_gtk_text_buffer_create_tag,V),
   MAKE_INTRINSIC_0("gtk_text_buffer_insert_with_tags",
@@ -1097,9 +1142,6 @@ static SLang_Intrin_Var_Type Global_Intrin_Vars[] =
    SLANG_END_INTRIN_VAR_TABLE
 };
 
-
-static const unsigned int _gtk_version = 10000*GTK_MAJOR_VERSION+ 
-				100*GTK_MINOR_VERSION + GTK_MICRO_VERSION;
 static SLang_Intrin_Var_Type Local_Intrin_Vars[] =
 {
 
